////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
Project 012 v105 - Weather Station: transmitter
CCS compiler
		
by Michel Bavin (c) May 6, 2006. --- info@techdesign.be --- www.techdesign.be ---  

... enjoy !

Open source for personal or educational use only.
*/
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <16f84a.h>			// device selection  

#fuses XT,NOWDT,PUT,NOPROTECT

#use delay(clock=4000000) 	// 4 MhZ  
#zero_ram 

#use fast_io(A)
#use fast_io(B)

#use rs232(baud=19200, xmit=PIN_B3, rcv=PIN_B4)

#define NOP #asm nop #endasm

#byte pa	=0x05	// port a	// 
#byte pb	=0x06	// port b	// 


// *********************************
// PORT A
// 
#bit lobat 			=pa.0		// RA0 pin17	// low battery detect
#bit tx_ena	 		=pa.1		// RA1 pin18	// power for TX433 module
#bit p_a2			=pa.2		// RA2 pin1		
#bit p_a3			=pa.3		// RA3 pin2
#bit p_a4			=pa.4		// RA4 pin3

// *********************************
// PORT B: 
// 
#bit p_b0 			=pb.0		// RB0 pin6		// INTERRUPT input
#bit tx_out 		=pb.1		// RB1 pin7		// output to TX433 module
#bit led			=pb.2		// RB2 pin8		// LED (active when sending) 
// #bit p_b3			=pb.3		// RB3 pin9		// RS232 TX
// #bit p_b4			=pb.4		// RB4 pin10	// RS232 RX
#bit TC77_SCK		=pb.5		// RB5 pin11	// SPI SCK
#bit TC77_SIO		=pb.6		// RB6 pin12	// SPI in/out
#bit TC77_CS		=pb.7		// RB7 pin13	// SPI !CS


// *********************************

//
void sleep_routine(void);						// for sleep mode
static short sleep_mode;
static char tosleep_counter;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#INT_EXT 										// wake up from BO / INT0
void ext_isr(void)				
{
static short button_pressed=FALSE;	

if(button_pressed){sleep_mode=FALSE;tosleep_counter=0;}
else{sleep_mode=TRUE;}

if(input(PIN_B0)){button_pressed=TRUE;}				
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



//													
void tc77_read(void);					// fot SPI temperature sensor TC77
signed int16 t_in,t_unused;
char tc77_ready;
void tc77_write(short bitfortc77);		
//

//
void send_tx433(void);					// fot sending routine (TX433 module)
int32 tx_temp;
//


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void main()
{

SET_TRIS_A(0x01);			// oooo oooi
SET_TRIS_B(0x11);			// oooi oooi

TC77_CS=1; 					// deselect the TC77 (RB7)
tx_ena=1;					// power-off for the TX433 module

sleep_mode=FALSE;			// init sleep flag
tosleep_counter=0;


printf("012 - Weather Station: Remote Sensor \n\r");	// RS232 test


// *********************************
while(TRUE){



tc77_read();

send_tx433();			// sending routine 


// while (!lobat){delay_ms(200);}		// low battery detect

sleep_routine();


			}	// end while



}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void sleep_routine(void)
{

if(tosleep_counter>1){sleep_mode=TRUE;}

if(sleep_mode){          // if sleep flag set
		
		ext_int_edge(L_TO_H);      			// init interrupt triggering for button press
		
		enable_interrupts(INT_EXT);			// turn on interrupts
		enable_interrupts(GLOBAL);

		tosleep_counter=0;
		
		NOP;NOP;NOP;NOP;

		sleep();            				// make processor sleep
		
		NOP;NOP;NOP;NOP;
					
				} 

if(!sleep_mode){ 
		
		disable_interrupts(INT_EXT);		// turn off interrupts
		disable_interrupts(GLOBAL);
 		
		tosleep_counter++;              		// sleep counter increment
		
					}


}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



void tc77_read(void){				// emulated SPI read

char b;
						
//tc77_write(0);					// power-up the tc77

SET_TRIS_B(0x51);				// oioi oooi
tc77_ready=0;					// reset

while (!tc77_ready){

	TC77_CS=0;						// select (chip enable)
	t_in=0;

	for(b=16;b!=0;b--){  			// 16-bit read
		TC77_SCK=0;
		delay_ms(2);
		TC77_SCK=1;
		if (TC77_SIO){bit_set(t_in,b-1);}
		delay_ms(2);
						}
	TC77_CS=1;											// deselect (chip disable)
	if(bit_test(t_in,2)==1){tc77_ready=1;break;}		// status of adc (tc77 ready?) takes 300mS after each power up
						
					}

t_in&=0xfff8; 									// reset lowest 3 bits

//tc77_write(1);					// power-down the tc77 (consumes only 0,1A in shutdown-mode)
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void tc77_write(short bitfortc77){

char b;
t_unused=0;
SET_TRIS_B(0x51);		// oioi oooi


TC77_CS=0;						// select (chip enable)
tc77_ready=0;
while (!tc77_ready){
	for(b=16;b!=0;b--){  		// 16-bit read
		TC77_SCK=0;
		delay_ms(2);
		TC77_SCK=1;
		if (TC77_SIO){bit_set(t_unused,b-1);}
		delay_ms(2);		
				}

	tc77_ready=bit_test(t_unused,2);				// status of adc (tc77 ready?) takes 300mS after each power up
					}


SET_TRIS_B(0x11);		// oooi oooi

if (!bitfortc77){TC77_SIO=0;}
else if (bitfortc77){TC77_SIO=1;}


for(b=16;b!=0;b--){  			// 16-bit write
		TC77_SCK=0;
		delay_ms(2);
		TC77_SCK=1;
		delay_ms(2);
					}

TC77_SCK=0;
TC77_CS=1;						// deselect (chip disable)
SET_TRIS_B(0x51);		// oioi oooi
}




////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void send_tx433(void)
{
char z,y;
int32 rest;
tx_out=0;
tx_ena=0;				// power on TX433 module
led=1;					// lit up LED when sending through TX433 module
delay_ms(10);
tx_out=1;

// tx_temp=0x00045201;				// test for 45.2C
// t_in=(signed int16)0xf387;		// test for -25.0C

tx_temp=0x00000001;

printf("t_in %lx\n\r",t_in);	// RS232 test

// signed int16 t_in to int32 tx_temp (BCD) conversion

// signed int16 t_in;
// int32 tx_temp;
// MSB 0x00ffff00 LSB
// 0 to 31
// bit23= lobat 	
// bit21= minus
// bits 23..20		(BCD3)
// bits 19..16	MSB (BCD2)
// bits 15..12		(BCD1)
// bits 11..8	LSB (BCD0)



// if (!bit_test(t_in,15)){;}	// if positive C
if (bit_test(t_in,15)){			// if negative C

t_in=~t_in;				// invert bits (from signed int16)
t_in+=8;				// correction

bit_set(tx_temp,21);	// set bit for negative C
						}


while(t_in>=0x0500){t_in-=0x0500;tx_temp+=0x00010000;}		// convert to BCD
while(t_in>=0x0080){t_in-=0x0080;tx_temp+=0x00001000;}
t_in>>=3;			// 3 LSBs not used by tc77
rest=(int32)t_in*0.625;	// 0.0625*10
rest<<=8;
tx_temp+=rest;


if (!lobat){bit_set(tx_temp,23);}


printf("tx_temp %Lx\n\r\n\r",tx_temp);	// RS232 test

tx_out=0;delay_ms(10);
tx_out=1;delay_us(1600);												// pre-1= 1900uS
for (z=0;z!=11;z++){tx_out=0;delay_us(1100);tx_out=1;delay_us(1600);}	// pre0=  (11*) !1000uS + 1900uS
tx_out=0;delay_us(4200);												// pre+1= !4200uS
tx_out=1;delay_us(5800);												// pre+2= 5800uS
tx_out=0;delay_us(4500);												// pre+3= !4500uS



for (y=0;y!=32;y++){if (bit_test(tx_temp,y)){tx_out=1;delay_us(1600);tx_out=0;delay_us(1100);}			// digital 1= 1900uS + !1000uS
					else if (!bit_test(tx_temp,y)){tx_out=0;delay_us(1100);tx_out=1;delay_us(1600);}	// digital 0= !1000uS + 1900uS			
					}
tx_out=0;
delay_ms(10);
tx_ena=1;				// power off TX433 module
led=0;

}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//													The End !!!!
